/*  ----------------------------------------------------------------------

    Copyright (C) 2000  Gabriel Gomiz  (gomita@bblanca.com.ar)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    ---------------------------------------------------------------------- */

    /* $Id$ */

#include "nomadii.h"

#ifdef PACKET_DUMP
int opt_debug();
#endif

char	quithelp[]     = "exit program";
char    helphelp[]     = "get help on commands";
char    openhelp[]     = "open Nomad II device";
char    closehelp[]    = "close Nomad II device";
char    modelhelp[]    = "get device model and firmware information";
char    ownerhelp[]    = "get/set owner information \n\
	owner [get]       - shows current device owner \n\
	owner set <name>  - sets device owner to <name>";
char    stathelp[]     = "show current status";
char    memhelp[]      = "show flash memory capacity status";
char    counthelp[]    = "number of files in flash memory";
char    listhelp[]     = "list flash memory contents";
char    versionhelp[]  = "display version information";
char    formathelp[]   = "format flash memory";
char    presetshelp[]  = "get/set radio presets \n\
	presets	                    - display the radio presets\n\
	presets read                - reload all presets from device\n\
	presets write               - write all presets to device \n\
	presets set all|<n> <f1f2>  - set preset <n> to f1.f2MHz";
char    clockhelp[]    = "get/set device clock \n\
	clock get   - display date and time from Nomad II \n\
	clock set   - set Nomad II date and time to now";
char    synchelp[]     = "synchronize memory from device";
char    downloadhelp[] = "put one or more MP3 files into flash memory\n\
		synonym of put and send";
char    sendhelp[] = "put one or more MP3 files into flash memory\n\
		synonym of put and download";
char    puthelp[] = "put one or more MP3 files into flash memory\n\
		synonym of send and download";
char    playlisthelp[] = "send playlist contents to Nomad II \n\
        playlist <file> [offset [number]]";
char    gethelp[]   = "get MP3 files <n> ... or \"all\" from flash memory\n\
		synonym of upload";
char    uploadhelp[]   = "get MP3 files <n> ... or \"all\" from flash memory\n\
		synonym of get";
char    deletehelp[]   = "delete files <n> ... or \"all\" from flash memory";
char    lcdhelp[]      = "print/change current working directory";
char    lpwdhelp[]      = "print current working directory";
char    autorenamehelp[]  = "turn underscores into spaces and remove .mp3 suffix in list";
char    verbosehelp[]  = "display more progress messages";
char    bellhelp[]     = "make sounds after completing transfers";
char    hashhelp[]     = "show progress during transfers";
char    prompthelp[]   = "interactive mode for playlist command";
char    forcehelp[]    = "do not ask for confirmation on delete/format";
char    internalhelp[] = "use internal flash memory";
char    externalhelp[] = "use external flash memory";
char    bothhelp[]   =   "use internal and external flash memory";
char    probehelp[]    = "reprobe USB for a Nomad II";

#ifdef PACKET_DUMP
char    debughelp[]    = "cycle debug level between off, 1 and 2 - ONLY FOR DEBUGGING";
#endif

struct cmd cmdtab[] = {
	{ "quit",       quithelp,       quit         },
	{ "bye",        quithelp,       quit         },
	{ "exit",       quithelp,       quit         },
	{ "help",       helphelp,       help         },
	{ "status",     stathelp,       status       },
	{ "open",       openhelp,       dev_open     },
	{ "close",      closehelp,      dev_close    },
	{ "model",      modelhelp,      model        },
	{ "owner",      ownerhelp,      owner        },
	{ "memory",     memhelp,        memory       },
	{ "count",      counthelp,      count        },
	{ "list",       listhelp,       list         },
	{ "ls",         listhelp,       list         },
	{ "version",    versionhelp,    version      },
	{ "format",     formathelp,     format       },
	{ "presets",    presetshelp,    presets      },
	{ "clock",      clockhelp,      dev_clock    },
	{ "sync",       synchelp,       memsync      },
	{ "download",   downloadhelp,   download     },
	{ "put",	puthelp,	download     },
	{ "send",	sendhelp,	download     },
	{ "playlist",   playlisthelp,   playlist     },
	{ "upload",     uploadhelp,     upload       },
	{ "get",	gethelp,        upload       },
	{ "delete",     deletehelp,     delete       },
	{ "lcd",        lcdhelp,        lcd          },
	{ "lpwd",       lpwdhelp,       lcd          },
	{ "autorename", autorenamehelp, opt_autorename },
	{ "verbose",    verbosehelp,    opt_verbose  },
	{ "bell",       bellhelp,       opt_bell     },
	{ "hash",       hashhelp,       opt_hash     },
	{ "prompt",     prompthelp,     opt_prompt   },
	{ "force",      forcehelp,      opt_force    },
	{ "internal",   internalhelp,   opt_internal },
	{ "external",   externalhelp,   opt_external },
	{ "both",       bothhelp,       opt_both     },
	{ "probe",      probehelp,      dev_reprobe  },
#ifdef PACKET_DUMP
	{ "debug",      debughelp,      opt_debug    },
#endif
	{ 0 },
};

int	NCMDS = sizeof (cmdtab) / sizeof (cmdtab[0]);
