#!/bin/sh

# $Id$

DEFINSTALLDIR=/usr/local/bin
NOMADPATH=`which nomadii 2>/dev/null`
HOTPLUGDIR=/etc/hotplug
MAPFILE=$HOTPLUGDIR/usb.usermap

NOMADIIENTRY="nomadII 0x0003 0x041e 0x1002"
NOMADIIMGENTRY="nomadII 0x0003 0x041e 0x4004"
MAPREST="0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00"

if [ `id -u` -ne 0 ]
then
    echo "this script must be run as root"
    exit 1
fi

if [ ! -f /proc/sys/kernel/hotplug ]
then
    echo "you do not appear to have kernel hotplug support"
    exit 1
fi

if [ `cat /proc/sys/kernel/hotplug` != "/sbin/hotplug" ]
then
    echo "/proc/sys/kernel/hotplug does not contain /sbin/hotplug"
    echo "as expected - this setup may not work"
fi

setupXUtil() {
    if [ ! -f nomadII_xterm ]
    then
	echo "missing nomadII_xterm or nomadII scripts"
	exit 1
    fi

    echo -n "install dir for nomadII_xterm script [$DEFINSTALLDIR]: "
    read INSTALLDIR

    if [ -z "$INSTALLDIR" ]
    then
	INSTALLDIR=$DEFINSTALLDIR
    fi

    if [ ! -d "$INSTALLDIR" ]
    then
	echo "not a valid directory: $INSTALLDIR"
	exit 1
    fi

    # try again if not as nomadii
    if [ ! -x "$NOMADPATH" ]
    then
	NOMADPATH=`which nomad 2>/dev/null`
    fi

    # then test for real
    if [ ! -x "$NOMADPATH" ]
    then
	echo "Nomad II control program as 'nomadii' or 'nomad'"
	echo -n "enter name of program: "
	read NAME

	NOMADPATH=`which $NAME 2>/dev/null`

	if [ ! -x "$NOMADPATH" ]
	then
	    echo "could not find program: $NAME"
	    exit 1
	fi
    fi
}

echo "Choose one of the hotplug options below:"
echo "  1) open full read/write permissions on nomad device"
echo "  2) change owner of nomad device to user on main console"
echo "  3) as 2, but also launch nomadii session in XTerm"
echo -n "Enter number: "

read OPTION

case $OPTION in
    1) LINE1='chmod 666 $DEVICE' ;;
    2) LINE1='chown --reference=/dev/console $DEVICE' ;;
    # all this nasty quoting is to get the damn $`' marks protected
    # so we can drop the whole line into a file later
    3) setupXUtil
       sed s#NOMADPATH#$NOMADPATH# < nomadII_xterm > $INSTALLDIR/nomadII_xterm
       chmod +x $INSTALLDIR/nomadII_xterm 
       LINE1='USER=`ls -l /dev/console | awk '"'{print "'$3}'"'\`"
       LINE2='chown $USER $DEVICE'
       LINE3="exec su -c $INSTALLDIR/nomadII_xterm - \$USER"
       ;;
    *) echo "invalid selection: choose 1, 2 or 3"
       exit 1;;
esac

if [ ! -d $HOTPLUGDIR/usb ]
then
    mkdir -p $HOTPLUGDIR/usb
fi

# install hotplug script, substituting in variables
cat > $HOTPLUGDIR/usb/nomadII <<EOM
#!/bin/sh

$LINE1
$LINE2
$LINE3
EOM
chmod +x $HOTPLUGDIR/usb/nomadII

# append map entry files if they are not already present
if ! grep -s "^$NOMADIIENTRY" $MAPFILE > /dev/null
then
    echo "$NOMADIIENTRY $MAPREST" >> $MAPFILE
fi

if ! grep -s "^$NOMADIIMGENTRY" $MAPFILE > /dev/null
then
    echo "$NOMADIIMGENTRY $MAPREST" >> $MAPFILE
fi
