/*  ----------------------------------------------------------------------

    Copyright (C) 2000  Gabriel Gomiz  (gomita@bblanca.com.ar)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    ---------------------------------------------------------------------- */

    /* $Id$ */

#include "nomadIIpriv.h"

#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <libgen.h>

#include <linux/usb.h>
#include <linux/usbdevice_fs.h>

#define DOWNLOAD_INTERVAL  500

static int nomadII_send_block(int fd, BYTE *buffer,int count)
{
	/* CONTROL 0x05 --> DOWNSTREAM (IN)                      */
	/* CONTROL <-- UPSTREAM (IN) (length 0x01) (00)          */
	/* BULK --> DOWNSTREAM (OUT) (length count) (data block) */
	/* BULK <-- UPSTREAM   (OUT)                             */
	/* CONTROL 0xF0 --> DOWNSTREAM (IN)                      */
	/* CONTROL <-- UPSTREAM (IN) (length 0x01) (00)          */

	if (nomadII_send_command(fd,COMMAND_SEND_BLOCK,count,0x00) != 0)
		return -1;

	if (nomadII_write(fd,buffer,count) != 0)
		return -1;

	if (nomadII_send_command(fd,COMMAND_GET_RESULT,0x00,0x00) != 0)
		return -1;

	return 0;
}

int nomadII_download_file(NomadIIHandle nomadII, int bank, char *file_name, int *bank_used) 
{
  	int src_file;
	int count,bytes_left;
	struct stat file_stat;
	BYTE buffer[DOWNLOAD_BLOCK_SIZE];
	BYTE status[2];
	unsigned int i,j,file_chars,buffer_size;
	WORD file_mtime, file_mdate;
	int progress, accum_step;
	float accum;
	int errno,result;
	DWORD file_size;
	char short_file_name[12];
	char *base_name;
	BYTE command[12] = { 0x00, 0x00, 0x00,   
			   0x00, 0x00, 0x00,
			   0x00, 0x00, 0x00,
			   0x00, 0x00, 0x00  };
	BYTE buffer_query[PATH_MAX*2+11];
	nomadII_file *file = &nomadII->files[nomadII->file_count];

	CHECKOPEN(nomadII);

	base_name = (char *)basename(file_name);

	if (nomadII_memory(nomadII) != 0)
		return -1;

	if (nomadII->file_count == MAX_FILES) {
		errno = EINVAL;
		nomadII_set_error("error: total number of files (%d) exceeds hardcoded maximum (%d).",
			nomadII->file_count, MAX_FILES);
		return -1;
	}
	
	file->type = FILE_ANY;

	if ((src_file = open(file_name,O_RDONLY)) < 0) {
		nomadII_set_error("reading file - %s", strerror(errno));
		return -1;
	}

	if (fstat(src_file,&file_stat) == -1) {
		nomadII_set_error("getting file information - %s", strerror(errno));
		return -1;
	}
	
	file->file_size = file_size = file_stat.st_size;	
	
	if (bank == BOTH) {
		if (file_size <= nomadII->mem_free[INTERNAL])
			bank = INTERNAL;
		else
			bank = EXTERNAL;
	}

	file->bank = *bank_used = bank;

	/* File name stuff */
	
	if (nomadII_longname_exists(nomadII, base_name, bank) ||
	    nomadII_build_dos_name(nomadII, short_file_name, base_name, bank, NULL) == -1) {
		errno = EEXIST;
		nomadII_set_error("a file of that name already exists");
		return -1;
	}

	/* CONTROL 0x25 --> DOWNSTREAM (IN)                */
	/* CONTROL <-- UPSTREAM (IN) (length 0x02) (01 00) */
	if (nomadII_read_command(nomadII->fd,COMMAND_BEGIN_TRANSFER,0x00,0x00,0x02,status) != 0) {
		close(src_file);
		return -1;
	}

	if (status[0] != 0x00 || status[1] != 0x01) {
		if (status[0] == 0x00 && status[1] == 0x00)
			nomadII_set_error("battery too low for download.");
		errno = EIO;
		close(src_file);
		return -1;
	}


	/* CONTROL 0x03 val 0x01 --> DOWNSTREAM (IN)             */
	/* CONTROL <-- UPSTREAM (IN) (length 0x0C) (data size)   */
	/* BULK --> DOWNSTREAM (OUT) (length 0x17) (data name)   */
	/* BULK <-- UPSTREAM   (OUT)                             */
	/* CONTROL 0xF0 --> DOWNSTREAM (IN)                      */
	/* CONTROL <-- UPSTREAM (IN) (length 0x01) (00)          */

	/* Load the file size */
	memcpy(&command[0x08],(void *)&file_size,sizeof(DWORD));

	/* set the modification date */
	nomadII_date_unix2dos(file_stat.st_mtime, &file_mdate, &file_mtime);
	memcpy(&command[0x06],(void *)&file_mdate,sizeof(WORD));
	memcpy(&command[0x04],(void *)&file_mtime,sizeof(WORD));	
	file->mtime = file_stat.st_mtime;

	/* Load size of name of file */
	file_chars = strlen(base_name)*2+1;
	memcpy(&command[2],(BYTE *)&file_chars,sizeof(BYTE));

	if (nomadII_write_command(nomadII->fd,COMMAND_SELECT_FILE,bank,0x00,0x0C,command) != 0) {
		close(src_file);
		return -1;
	}

	strcpy((char *)buffer_query,short_file_name);

	buffer_size = 11+file_chars;

	for (i = 0, j = 11; j < buffer_size; i++, j+=2) {
		buffer_query[j] = *(base_name+i);
		buffer_query[j+1] = 0;
	} 

	if (nomadII_write(nomadII->fd,buffer_query,11+file_chars) != 0) {
		close(src_file);
		return -1;
	}

	/* CONTROL 0xF0 --> DOWNSTREAM (IN)             */
	/* CONTROL <-- UPSTREAM (IN) (length 0x01) (00) */
	
	if ((result = nomadII_send_command(nomadII->fd,COMMAND_GET_RESULT,0x00,0x00)) != 0) {
		switch (result) {

		case RESULT_NO_SPACE_LEFT:
			errno = ENOMEM;
			break;
		
		case RESULT_FILE_EXISTS:
			errno = EEXIST;
			break;
			
		default:
			errno = EIO;
		}		
		close(src_file);
		return -1;
	}

	/*
	 * Block transfers
	 */

	/* progress - nomadII_progress_ticks ticks */
	accum    = 0.01; /* prime to cope with rounding error */
	progress = 0;

	bytes_left = file_size;
  
	if (nomadII_progress_handler)
		nomadII_progress_handler(NOMADII_PROGRESS_NEW_FILE, 0, nomadII_progress_data);
		
	while (bytes_left > 0) {
		count = read(src_file,buffer,DOWNLOAD_BLOCK_SIZE);
		
		if (count == -1 || nomadII_send_block(nomadII->fd,buffer,count) != 0) {
			close(src_file);
			return -1;
		}
		
		if (nomadII_progress_handler) {
			/* proportion completed out of nomadII_progress_ticks */
			accum     += (float)(count*nomadII_progress_ticks)/(float)file_size;
			accum_step = (int)accum;
		
			while (accum_step > progress)
				nomadII_progress_handler(NOMADII_PROGRESS_UPDATE, progress++, nomadII_progress_data);
		}
			
		bytes_left -= count;
		usleep(DOWNLOAD_INTERVAL);
  
	}

	if (nomadII_progress_handler && progress <= nomadII_progress_ticks)
		nomadII_progress_handler(NOMADII_PROGRESS_UPDATE, nomadII_progress_ticks, nomadII_progress_data);
	
	/* CONTROL 0x04 --> DOWNSTREAM (IN)             */
	/* CONTROL <-- UPSTREAM (IN) (length 0x01) (00) */

	if (nomadII_send_command(nomadII->fd,COMMAND_END_TRANSFER,0x00,0x00) != 0) {
		close(src_file);
		return -1;
	}

	/* this info will be wiped on a sync, but must be present for
	   duplicate name checking to work properly */
	nomadII_set_file_name(short_file_name, file->file_name);
	nomadII_set_file_ext(short_file_name+8, file->file_ext);
	nomadII_set_long_file_name(base_name, file->long_file_name);
	nomadII->file_count++;	

	if (nomadII_memory(nomadII) != 0) {
		close(src_file);
		return -1;
	}

	close(src_file);
	
	return 0;
}
