/*  ----------------------------------------------------------------------

    Copyright (C) 2002  Tom Mander <tom@who.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    ---------------------------------------------------------------------- */

    /* $Id$ */

#include "nomadIIpriv.h"
#include <time.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <stdarg.h>

char				nomadIIErrorBuff[MAX_ERROR_LENGTH];
nomadII_progress_handler_cb*	nomadII_progress_handler = NULL;
void*				nomadII_progress_data;
int				nomadII_progress_ticks;

char* nomadII_get_library_version()
{
	return NOMADII_LIB_VERSION;
}

char* nomadII_get_error()
{
	if (nomadIIErrorBuff[0] == '\0')
		return strerror(errno);
	else
		return nomadIIErrorBuff;
}

void nomadII_set_progress_handler(nomadII_progress_handler_cb* handler, void* data, int ticks)
{
	nomadII_progress_handler = handler;
	nomadII_progress_data    = data;
	nomadII_progress_ticks   = ticks;
}

void nomadII_set_error(char* format, ...)
{
	va_list arglist;

	va_start(arglist, format);
	vsnprintf(nomadIIErrorBuff, MAX_ERROR_LENGTH, format, arglist);
	va_end(arglist);
}

/* tests (case-insensitive) for long filename in syncronized file list */
int nomadII_longname_exists(NomadIIHandle nomadII, char* long_name, int bank)
{
	int i;
	
	for (i = 0; i < nomadII->file_count; i++)
		if (nomadII->files[i].bank == bank &&
		    strcasecmp(long_name, nomadII->files[i].long_file_name) == 0)
			return 1;
	return 0;
}

/* capitalize and substitute illegal characters */
static void dossify(char* buff)
{
	char *p = buff;

	while (*p != '\0') {
		/* suppress illegal DOS chars */
		if (strchr("/\\[]:*|+=;,?.", *p) != NULL)
			*p = '_';
		else
			*p = toupper(*p);
		p++;
	}	
}

/* turn spaces and non-chars to underscores */
static void despace(char* buff, int length)
{
	char *p = buff;

	while (length--) {
		if (*p == ' ' || *p < 0x20 || *p > 0x7E)
			*p = '_';
		p++;
	}	
}

/* tests for 8.3 filename in syncronized file list */
static int shortname_exists(NomadIIHandle nomadII, char* short_name, int bank)
{
	int i;
	
	for (i = 0; i<nomadII->file_count; i++)
		if (nomadII->files[i].bank == bank &&
		    strncasecmp(short_name, nomadII->files[i].file_name, 8) == 0 &&
		    strncasecmp(&short_name[8], nomadII->files[i].file_ext, 3) == 0)
			return 1;
	
	return 0;
}

/* encodes long name as short DOS style name FFFFFF~N.EXT if longer than 11 chars
   - returns -1 if short name cannot be created to be unique on the given memory bank */
int nomadII_build_dos_name(NomadIIHandle nomad, char* out_file_name, char* in_file_name, int bank, char* retsuffix)
{
	int  i, unique_len, suffix_len = 0, in_len = strlen(in_file_name);
   	char *suffix;
   	char unique[9];
                
	suffix = strrchr(in_file_name, '.');
        
	if (suffix != NULL)
		suffix_len = strlen(suffix++);

	in_len -= suffix_len;
	if (suffix_len > 0) {
		suffix_len--;
	} else {
		/* Nomad needs a suffix */
		suffix = "_";
		suffix_len = 1;
	}


	/* fields a padded with spaces */
	if (in_len == 0) {
		strncpy(out_file_name, "~1        ", 8);
		strncpy(out_file_name + 8, suffix, suffix_len);
		despace(out_file_name + 8, suffix_len);
		strncpy(out_file_name + suffix_len + 8, "   ", 3 - suffix_len);
		out_file_name[11] = '\0';
		dossify(out_file_name);
	
		if (shortname_exists(nomad, out_file_name, bank)) {
			errno = EEXIST;
			return -1;
		}
	} else if (in_len <= 8) {
		strncpy(out_file_name, in_file_name, in_len);
		despace(out_file_name, in_len);
		strncpy(out_file_name + in_len, "        ", 8 - in_len);
		strncpy(out_file_name + 8, suffix, suffix_len);
		despace(out_file_name + 8, suffix_len);
		strncpy(out_file_name + suffix_len + 8, "   ", 3 - suffix_len);
		out_file_name[11] = '\0';
		dossify(out_file_name);
		if (shortname_exists(nomad, out_file_name, bank)) {
			errno = EEXIST;
			return -1;
		}
	} else {
		strncpy(out_file_name, in_file_name, 6);
		despace(out_file_name, 6);
		strncpy(out_file_name + 6, "~1", 2);
		strncpy(out_file_name + 8, suffix, suffix_len);
		despace(out_file_name + 8, suffix_len);
		strncpy(out_file_name + suffix_len + 8, "   ", 3 - suffix_len);
		out_file_name[11] = '\0';
		dossify(out_file_name);
		
		i = 2;
		while (shortname_exists(nomad, out_file_name, bank)) {
			if (i >= 100000) {
				errno = EEXIST;
				return -1;	
			}
			
			snprintf(unique, 9, "~%d", i++);
			unique_len = strlen(unique);
			strncpy(out_file_name + 8 - unique_len, unique, unique_len);
		}
	}
	
	if (retsuffix != NULL)
		/* make sure we use capitalized, non-NULL suffix */
		strncpy(retsuffix, out_file_name + 8, 4);
	return 0;
}

/* The following functions are derived from those from msdos-fs */

/* Linear day numbers of the respective 1sts in non-leap years. */
static int day_n[] =
{0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 0, 0, 0, 0};
/* Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec */

/* Convert a MS-DOS time/date pair to a epoch UNIX time (seconds since 1 1 70). */

time_t nomadII_date_dos2unix(WORD dosdate, WORD dostime)
{
	int month, year;
	time_t secs;
	static time_t now = 0;
	
	if (now == 0) {
		now = time(NULL);
		localtime(&now); /* initialize __timezone */
	}

	month = ((dosdate >> 5) & 15) - 1;
	year = dosdate >> 9;
	secs = (dostime & 31) * 2 + 60 * ((dostime >> 5) & 63) + (dostime >> 11) * 3600 + 86400 *
	    ((dosdate & 31) - 1 + day_n[month] + (year / 4) + year * 365 - ((year & 3) == 0 &&
						   month < 2 ? 1 : 0) + 3653);
	/* days since 1.1.70 plus 80's leap day */
	return secs + __timezone;
}


/* Convert epoch UNIX time to a MS-DOS time/date pair. */

void nomadII_date_unix2dos(int unix_date, WORD *dosdate, WORD *dostime)
{
	int day, year, nl_day, month;
	static time_t now = 0;
	
	if (now == 0) {
		now = time(NULL);
		localtime(&now); /* initialize __timezone */
	}

	unix_date -= __timezone;
	*dostime = (unix_date % 60) / 2 +
		(((unix_date / 60) % 60) << 5) +
		(((unix_date / 3600) % 24) << 11);

	day = unix_date / 86400 - 3652;
	year = day / 365;
	if ((year + 3) / 4 + 365 * year > day)
		year--;
	day -= (year + 3) / 4 + 365 * year;
	if (day == 59 && !(year & 3)) {
		nl_day = day;
		month = 2;
	} else {
		nl_day = (year & 3) || day <= 59 ? day : day - 1;
		for (month = 0; month < 12; month++)
			if (day_n[month] > nl_day)
				break;
	}
	*dosdate = nl_day - day_n[month - 1] + 1 + (month << 5) + (year << 9);
}


/*
 * Filename functions
 */
void nomadII_set_file_name(char *src, char *dst) 
{
	int i;

	for (i = 0; i < 8; i++)
		*(dst+i) = *(src+i);
	*(dst+8) = '\0';
}

void nomadII_set_file_ext(char *src, char *dst) 
{
	int i;
	
	for (i = 0; i < 3; i++)
		*(dst+i) = *(src+i);
	*(dst+3) = '\0';
}

void nomadII_set_long_file_name(char *src, char *dst) 
{
	int i,j;

	for (i = 0, j = 0; *(src+i) != '\0'; i+=2, j++)
		*(dst+j) = *(src+i);
	*(dst+j) = '\0';
}

/* based on zlib/adler32.c -- compute the Adler-32 checksum of a data stream
 * Copyright (C) 1995-1998 Mark Adler
 * For conditions of distribution and use, see copyright notice in zlib.h 
 */

#define BASE 65521L /* largest prime smaller than 65536 */
#define NMAX 5552
/* NMAX is the largest n such that 255n(n+1)/2 + (n+1)(BASE-1) <= 2^32-1 */

#define DO1(buf,i)  {s1 += buf[i]; s2 += s1;}
#define DO2(buf,i)  DO1(buf,i); DO1(buf,i+1);
#define DO4(buf,i)  DO2(buf,i); DO2(buf,i+2);
#define DO8(buf,i)  DO4(buf,i); DO4(buf,i+4);
#define DO16(buf)   DO8(buf,0); DO8(buf,8);

/* ========================================================================= */
unsigned long nomadII_adler32(const BYTE* buf, int len)
{
    unsigned long s1 = 0;
    unsigned long s2 = 0;
    int k;

    if (buf == NULL) return 1L;

    while (len > 0) {
        k = len < NMAX ? len : NMAX;
        len -= k;
        while (k >= 16) {
            DO16(buf);
	    buf += 16;
            k -= 16;
        }
        if (k != 0) do {
            s1 += *buf++;
	    s2 += s1;
        } while (--k);
        s1 %= BASE;
        s2 %= BASE;
    }
    return (s2 << 16) | s1;
}

#ifdef PACKET_DUMP

int nomadII_hex_dump(char *data, long size)
{
	long i;
	long x;
	int r=1;
	char d[32];
	unsigned char bfr[64];

#ifdef MAX_DUMP_SIZE
	int trunc = size > MAX_DUMP_SIZE;
	int orig_size = size;
	size = (trunc ? MAX_DUMP_SIZE : size);
#endif

	for(i=0, x=0;;i++) {

		if(i<size) {
			if(x==0)
				fprintf(stdout,"* %04lx: ", i);
			snprintf(d, 9, "%08x", data[i]);
			fprintf(stdout,"%c%c ", d[6], d[7]);
			bfr[x]=data[i];
			if(bfr[x]<0x20)
				bfr[x]='.';
			if(bfr[x]>0x7f)
				bfr[x]='.';
		}
		else {
			if(x==0)
				break;
			else
			{
				fprintf(stdout,"   ");
				bfr[x]=' ';
				r=0;
			}
		}
		x++;
		if(!(x<16)) {
			bfr[x]=0;
			fprintf(stdout,"%s\n", bfr);
			x=0;
			if(!r)
				break;
		}
	}

#ifdef MAX_DUMP_SIZE
	if (trunc)
		fprintf(stdout,"* packet dump truncated, true size=0x%x\n", orig_size);
#endif

	fprintf(stdout,"*\n");
	return 0;
}

#endif /* PACKET_DUMP */
