/*  ----------------------------------------------------------------------

    Copyright (C) 2000  Gabriel Gomiz  (gomita@bblanca.com.ar)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    ---------------------------------------------------------------------- */

    /* $Id$ */

#ifndef NOMADII_CTL_H
#define NOMADII_CTL_H

#include <nomadII.h>

/* Megabytes/Kilobytes as a float */
#define MBF 1048576.0 
#define KBF 1024.0 

struct	cmd {
	char	*c_name;		/* command name */
	char	*c_help;		/* help message */
	int	(*c_handler)();		/* routine to do the work */
};

#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE  1
#endif

/* options constants */

#define OPTION_CLOCK_SET      10
#define OPTION_CLOCK          11
#define OPTION_OWNER_SET      12
#define OPTION_OWNER          13
#define OPTION_PRESETS_SET    14
#define OPTION_PRESETS        15
#define OPTION_FREQ           16
#ifdef  PACKET_DUMP
#define OPTION_DEBUG          17
#endif

extern int  autorename;  /* autorename when copying files */
extern int  interactive; /* interactive execution */
extern int  hash;        /* print progress bar on transfers */
extern int  bell;        /* make sound when tranfers finish */
extern int  prompt;      /* interactively prompt on playlist transfers */
extern int  force;       /* don't ask for confirmation on delete/format */
extern int  verbose;     /* speak a little more... */
extern int  memory_bank; /* which memory bank to use */
#ifdef PACKET_DUMP
extern int  debug;       /* show packet dump */
#endif

extern nomadII_device nomadII;

/* command prototypes */
void usage();
int help(int argc, char *argv[]);
void verb(char *msg);
int version();
int quit(int argc, char *argv[]);
int status();
int memsync();
int dev_probe_status(int silent);
int dev_reprobe();
void dev_probe();
int dev_open();
int dev_close();
int model();
int get_clock();
int set_clock();
int dev_clock(int argc, char *argv[]);
int get_owner();
int set_owner(char *owner);
int owner(int argc, char *argv[]);
int memory();
int count();
int list(int argc, char *argv[]);
int format();
int do_delete(int i);
int do_delete_all();
int delete(int argc, char *argv[]);
int show_presets();
int get_presets();
int set_preset(unsigned int preset, unsigned int freq, unsigned int subfreq);
int set_presets();
int presets(int argc,char *argv[]);
int do_download(char *file_name);
int download(int argc, char *argv[]);
int do_playlist(char *playlist_file_name, int offset, int number);
int playlist(int argc, char *argv[]);
int do_upload(int i);
int do_upload_all();
int upload(int argc, char *argv[]);
int lcd(int argc, char *argv[]);
int lcd(int argc, char *argv[]);
int opt_verbose();
int opt_autorename();
int opt_bell();
int opt_hash();
int opt_prompt();
int opt_force();
int opt_internal();
int opt_external();
int opt_both();
#ifdef PACKET_DUMP
int opt_debug();
#endif
int yesno();

void progress_handler(int, int, void*);

#endif /* NOMADII_CTL_H */
